require File.expand_path(File.dirname(__FILE__) + '/spec_helper')

describe "Duration" do

  before(:all) do
    @ms_in_a_second = 1000
    @ms_in_a_minute = @ms_in_a_second * 60
    @ms_in_an_hour = @ms_in_a_minute * 60
    @ms_in_a_day = @ms_in_an_hour * 24
    @ms_in_a_week = @ms_in_a_day * 7
    @ms_in_a_year = @ms_in_a_week * 52
  end

  describe "creation" do

    it "should know how many millis it was made with" do
      d = TickTock::Duration.new(4321)
      d.raw_millis.should == 4321
    end

    it "should use zero if no millis specified" do
      d = TickTock::Duration.new
      d.raw_millis.should == 0
    end

  end

  describe "#years" do

    it "should have 0 years when the duration has fewer than the number of millis in a year" do
      TickTock::Duration.new.years.should == 0
      TickTock::Duration.new(@ms_in_a_week * 51).years.should == 0
      TickTock::Duration.new(@ms_in_a_year - 1).years.should == 0
    end

    it "should have 1 year at the boundary" do
      TickTock::Duration.new(@ms_in_a_year).years.should == 1
      TickTock::Duration.new(@ms_in_a_year + @ms_in_a_week * 17).years.should == 1
    end

    it "should have 11 years if it's got a lot of millis" do
      TickTock::Duration.new(@ms_in_a_year * 11).years.should == 11
    end

  end

  describe "#weeks" do

    it "should have 0 weeks when the duration has fewer than the number of millis in a week" do
      TickTock::Duration.new.weeks.should == 0
      TickTock::Duration.new(@ms_in_a_day * 4).weeks.should == 0
      TickTock::Duration.new(@ms_in_a_week - 1).weeks.should == 0
    end

    it "should have 1 week when the duration is at the boundary" do
      TickTock::Duration.new(@ms_in_a_week).weeks.should == 1
    end

    it "should be able to handle multiple weeks less than a year" do
      TickTock::Duration.new(@ms_in_a_week * 30).weeks.should == 30
      TickTock::Duration.new(@ms_in_a_week * 51).weeks.should == 51
    end

    it "should account for a year if the duration is more than a year" do
      TickTock::Duration.new(@ms_in_a_year).weeks.should == 0
      TickTock::Duration.new(@ms_in_a_year + @ms_in_a_week - 1).weeks.should == 0
      TickTock::Duration.new(@ms_in_a_year + @ms_in_a_week).weeks.should == 1
    end

    it "should work with multiple years and multiple weeks" do
      tick_tock_duration = TickTock::Duration.new(@ms_in_a_year * 8 + @ms_in_a_week * 40)
      tick_tock_duration.weeks.should == 40
      tick_tock_duration.years.should == 8
    end

  end

  describe "#days" do

    it "should have 0 days when the duration has fewer than the number of millis in a day" do
      TickTock::Duration.new.weeks.should == 0
      TickTock::Duration.new(@ms_in_an_hour * 10).days.should == 0
      TickTock::Duration.new(@ms_in_a_day - 1).days.should == 0
    end

    it "should should have 1 day when at the day boundary" do
      TickTock::Duration.new(@ms_in_a_day).days.should == 1
    end

    it "should should have 1 day when just under the 2 day boundary" do
      TickTock::Duration.new(@ms_in_a_day * 2 - 1).days.should == 1
    end

    it "should work when just over a week" do
      duration = TickTock::Duration.new(@ms_in_a_week + 1)
      duration.weeks.should == 1
      duration.days.should == 0
    end

    it "should work with multiple weeks when just over a week" do
      duration = TickTock::Duration.new((@ms_in_a_week * 2) + (@ms_in_a_day * 3))
      duration.weeks.should == 2
      duration.days.should == 3
    end

    it "should work with multiple years, weeks, and days" do
      duration = TickTock::Duration.new((@ms_in_a_year * 5) + (@ms_in_a_week * 2) + (@ms_in_a_day * 3))
      duration.years.should == 5
      duration.weeks.should == 2
      duration.days.should == 3
    end

  end

  describe "#hours" do

    it "should have 0 hours when the duration has fewer millis than there are in an hour" do
      duration = TickTock::Duration.new(@ms_in_an_hour - 1)
      duration.hours.should == 0
    end

    it "should have 1 hour when the duration is exactly one hour in millis" do
      duration = TickTock::Duration.new(@ms_in_an_hour)
      duration.hours.should == 1
    end

    it "should have 23 hours when the duration is just under a day" do
      duration = TickTock::Duration.new(@ms_in_a_day - 1)
      duration.hours.should == 23
    end

    it "should have 0 hours when the duration is exactly a day" do
      duration = TickTock::Duration.new(@ms_in_a_day)
      duration.hours.should == 0
    end

    it "should have 0 hours when the duration is a day and up to 1 hour" do
      duration = TickTock::Duration.new(@ms_in_a_day + (@ms_in_an_hour - 1))
      duration.hours.should == 0
    end

    it "should have 1 hours when the duration is exactly a day and an hour" do
      duration = TickTock::Duration.new(@ms_in_a_day + (@ms_in_an_hour))
      duration.hours.should == 1
    end

  end

  describe "#minutes" do

    it "should have 0 minutes if millis are 0" do
      duration = TickTock::Duration.new(0)
      duration.minutes.should == 0
    end

    it "should have 0 minutes when just less than the boundary between 0 minutes and 1 minute" do
      duration = TickTock::Duration.new(@ms_in_a_minute - 1)
      duration.minutes.should == 0
    end

    it "should have 1 minute when it has exactly one minute of millis" do
      duration = TickTock::Duration.new(@ms_in_a_minute)
      duration.minutes.should == 1
    end

    it "should have 1 minute up to the 2 minute boundary" do
      duration = TickTock::Duration.new(@ms_in_a_minute * 2 - 1)
      duration.minutes.should == 1
    end

    it "should have 59 minutes up to the hour boundary" do
      duration = TickTock::Duration.new(@ms_in_an_hour - 1)
      duration.minutes.should == 59
    end

    it "should have 0 minutes at the hour boundary" do
      duration = TickTock::Duration.new(@ms_in_an_hour)
      duration.minutes.should == 0
    end

    it "should cross the hour boundary gracefully" do
      duration = TickTock::Duration.new(@ms_in_an_hour + 1)
      duration.hours.should == 1
      duration.minutes.should == 0
    end

  end

  describe "#seconds" do

    it "should have 0 seconds if millis are 0" do
      duration = TickTock::Duration.new(0)
      duration.seconds.should == 0
    end

    it "should have 0 seconds up until 1 second of millis" do
      duration = TickTock::Duration.new(999)
      duration.seconds.should == 0
    end

    it "should have 1 second at exactly 1 second of millis" do
      duration = TickTock::Duration.new(1000)
      duration.seconds.should == 1
    end

    it "should have 1 second up to 2 seconds of millis" do
      duration = TickTock::Duration.new(1999)
      duration.seconds.should == 1
    end

    it "should have 2 seconds after 2 seconds of millis" do
      duration = TickTock::Duration.new(2001)
      duration.seconds.should == 2
    end

  end

  describe "#millis" do

    it "should have 0 seconds if millis are 0" do
      duration = TickTock::Duration.new(0)
      duration.millis.should == 0
    end

    it "should count millis up to 1 second" do
      duration = TickTock::Duration.new(999)
      duration.millis.should == 999
    end

    it "should reset at 1 second" do
      duration = TickTock::Duration.new(1000)
      duration.millis.should == 0
    end

    it "should count between 1000 and 2000 millis" do
      duration = TickTock::Duration.new(1457)
      duration.millis.should == 457
    end

  end

  describe "all the bits at once" do

    it "should account for milliseconds, seconds, minutes, hours, days, weeks, and years" do
      duration = TickTock::Duration.new((@ms_in_a_year * 6) + (@ms_in_a_week * 17) + (@ms_in_a_day * 5) + (@ms_in_an_hour * 4) + (@ms_in_a_minute * 14) + (@ms_in_a_second * 45) + 973)
      duration.years.should == 6
      duration.weeks.should == 17
      duration.days.should == 5
      duration.hours.should == 4
      duration.minutes.should == 14
      duration.seconds.should == 45
      duration.millis.should == 973
    end

  end

  describe ".start_to_finish" do

    it "should allow creation from two times" do
      start = Time.utc(2012, 12, 10)
      finish = Time.utc(2012, 12, 10, 4)
      duration = TickTock::Duration.from_times(start, finish)
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 4
      duration.minutes.should == 0
      duration.seconds.should == 0
      duration.millis.should == 0
    end

  end


  describe ".from_pieces" do

    it "should allow creation from just millis" do
      duration = TickTock::Duration.from_pieces(10)
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 0
      duration.minutes.should == 0
      duration.seconds.should == 0
      duration.millis.should == 10
    end

    it "should allow creation from millis and seconds" do
      duration = TickTock::Duration.from_pieces(10, 40)
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 0
      duration.minutes.should == 0
      duration.seconds.should == 40
      duration.millis.should == 10
    end

    it "should allow creation from millis, seconds, and minutes" do
      duration = TickTock::Duration.from_pieces(10, 40, 20)
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 0
      duration.minutes.should == 20
      duration.seconds.should == 40
      duration.millis.should == 10
    end

    it "should allow creation from millis, seconds, minutes, and hours" do
      duration = TickTock::Duration.from_pieces(10, 40, 20, 5)
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 5
      duration.minutes.should == 20
      duration.seconds.should == 40
      duration.millis.should == 10
    end

    it "should allow creation from millis, seconds, minutes, hours, and days" do
      duration = TickTock::Duration.from_pieces(10, 40, 20, 5, 2)
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 2
      duration.hours.should == 5
      duration.minutes.should == 20
      duration.seconds.should == 40
      duration.millis.should == 10
    end

    it "should allow creation from millis, seconds, minutes, hours, days, and weeks" do
      duration = TickTock::Duration.from_pieces(10, 40, 20, 5, 2, 40)
      duration.years.should == 0
      duration.weeks.should == 40
      duration.days.should == 2
      duration.hours.should == 5
      duration.minutes.should == 20
      duration.seconds.should == 40
      duration.millis.should == 10
    end

    it "should allow creation from millis, seconds, minutes, hours, days, weeks, and years" do
      duration = TickTock::Duration.from_pieces(10, 40, 20, 5, 2, 40, 1023)
      duration.years.should == 1023
      duration.weeks.should == 40
      duration.days.should == 2
      duration.hours.should == 5
      duration.minutes.should == 20
      duration.seconds.should == 40
      duration.millis.should == 10
    end

    it "should overflow lesser bits into more significant bits" do
      duration = TickTock::Duration.from_pieces(2223, 70, 65, 5, 2, 40, 1023)
      duration.years.should == 1023
      duration.weeks.should == 40
      duration.days.should == 2
      duration.hours.should == 6
      duration.minutes.should == 6
      duration.seconds.should == 12
      duration.millis.should == 223
    end

  end

  describe "addition" do

    it "should support simple addition" do
      duration1 = TickTock::Duration.from_pieces(823)
      duration2 = TickTock::Duration.from_pieces(455)
      duration = duration1 + duration2
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 0
      duration.minutes.should == 0
      duration.seconds.should == 1
      duration.millis.should == 278
    end

    it "should support adding a bunch together" do
      duration1 = TickTock::Duration.from_pieces(10)
      duration2 = TickTock::Duration.from_pieces(0, 34)
      duration3 = TickTock::Duration.from_pieces(0, 0, 4)
      duration4 = TickTock::Duration.from_pieces(0, 0, 0, 10)
      duration = duration1 + duration2 + duration3 + duration4
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 10
      duration.minutes.should == 4
      duration.seconds.should == 34
      duration.millis.should == 10
    end

    it "should work with +=" do
      duration = TickTock::Duration.from_pieces(10)
      extra = TickTock::Duration.from_pieces(0, 34)
      duration += extra
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 0
      duration.minutes.should == 0
      duration.seconds.should == 34
      duration.millis.should == 10
    end

  end

  describe "subtraction" do

    it "should support simple subtraction" do
      duration1 = TickTock::Duration.from_pieces(823)
      duration2 = TickTock::Duration.from_pieces(455)
      duration = duration1 - duration2
      duration.years.should == 0
      duration.weeks.should == 0
      duration.days.should == 0
      duration.hours.should == 0
      duration.minutes.should == 0
      duration.seconds.should == 0
      duration.millis.should == 368
    end

  end

end
